Attribute VB_Name = "modSigGen"
'=======================================================================================
' Descrizione.....: Progetto per la generazione di segnali ad ampiezza, frequenza e
'                   forma d' onda voluta.
' Nome dei Files..: frmSigGen.frm, frmSigGen.frx,
'                   frmImpostazioni.frm, frmImpostazioni.frx,
'                   frmStampaFCPB.frm, frmStampaFCPB.frx,
'                   frmInfoCr.frm, frmInfoCr.frx,
'                   frmIstruzioni.frm, frmIstruzioni.frx,
'                   modSigGen.bas, modCommonDialog.bas,
'                   modMixerControl.bas, modUtilita.bas.
' Data............: 03/01/2000 (versione per FiltCalc).
' Aggiornamento...: 11/04/2001
' Aggiornamento...: 08/04/2004 (aggiunto lo "sweep").
' Aggiornamento...: 06/08/2005 (Ver. 2.0 versione per scheda audio con CallBack).
' Aggiornamento...: 01/10/2005 (Ver. 2.1 aggiunta la compensazione del ritardo
'                               del multiplexer).
' Aggiornamento...: 04/02/2007 (Ver. 2.2 aggiunto lo "sweep" bidirezionale).
' Aggiornamento...: 21/03/2007 (Ver. 2.2.1 eliminate alcune incongruenze della Ver. 2.2
'                               ed aggiunto il prodotto dei canali).
' Aggiornamento...: 04/04/2007 (Ver. 2.3 aggiunto il "duty cycle" dell' onda quadra,
'                               la simmetria dell' onda triangolare, la stampa dello
'                               strumento e l' "import/export" dell' onda grafica).
' Aggiornamento...: 31/08/2009 (Ver. 3.0 aggiunta la generazione del segnale
'                               senza "callback").
' Aggiornamento...: 31/10/2009 (Ver. 3.1 nuova versione delle routines di gestione
'                               delle schede audio, etc...).
' Versione........: 3.1 a 32 bits.
' Sistema.........: VB6 (SP5) sotto Windows XP (SP2).
' Scritto da......: F. Languasco 
' E-Mail..........: MC7061@mclink.it
' DownLoads a.....: http://www.flanguasco.org
'=======================================================================================
'
'------------------------------------------------------------------------------
'   Devo i miei piu' sentiti ringraziamenti a Marv Solowey, <mmsont@yahoo.ca>,
'   per avermi stimolato all' uso delle funzioni di "Callback", fornendomi pure
'   preziosi esempi.
'------------------------------------------------------------------------------
'
'   Nota 1: questo progetto, se compilato, deve essere compilato
'           in P-Code (vedi documento Microsoft q198607.pdf).
'
'   Nota 2: la funzione Rnd(), intrinseca del Visual Basic, non
'           funziona all' interno di "callback" quando il progetto
'           e' compilato: al "run time" da' errore di "Memory access
'           violation at ...".  L' ho, pertanto, sostituita con la
'           funzione Ran0() (vedi).
'
'   Nota 3: questo progetto e' stato sviluppato usando il "." come
'           separatore decimale; nonostante abbia cercato di prevenire
'           possibili problemi dovuti all' uso della "," non posso
'           escludere che sia rimasta qualche incongruenza.
'           In particolare potrebbe essere fonte di guai la lettura,
'           dal file SigGen.ini (specialmente con Windows XP!), dei pa-
'           rametri di calibrazione (vedere i valori assegnati nella
'           routine LeggiParametri e successivamente osservabili con
'           il menu [Impostazioni/Parametri del generatore]).
'           In caso di malfunzionamenti dovuti a questo problema,
'           provate a cambiare sul vostro PC le impostazioni interna-
'           zionali e datemi notizia dei "bugs" trovati.
'
'   Nota 4: in questo progetto molti controlli e variabili, comuni ai
'           due canali, sono indicizzati (1 To 2), con l' indice 1
'           riferito al canale sinistro (Ch. #1) ed il 2 al canale
'           destro (Ch. #2).
'
'   Nota 5: ho introdotto l' unita' di misura [DigVal] per indicare valori
'           digitali a 16 Bits, i.e. quelli direttamente inviati al
'           convertitore D/A della scheda audio.  L' unita' [Vu,p) indica
'           valori di cresta; l' unita' [Vu,pp) indica valori picco-picco.
'           L' unita' [campioni] e' per le grandezze misurabili in [s * Fs].
'
'   Nota 6: questa versione non puo' funzionare in modo "Run" per piu'
'           di (circa) 13 ore continuative.
'
Option Explicit
'
Public MixerSpI&        ' Indice, in MixerSp(), del mixer corrente di tipo SPEAKERS.
                        '  1 <= MixerSpI <= NMixSp.
'
Public Fs#                      ' Frequenza di campionamento [Hz].
Private Const NBuff& = 2        ' N di buffers per la generazione del segnale.
Public Const TBuff# = 0.4       ' Lunghezza dei buffers [s]; compromesso tra una
                                ' ragionevole lunghezza dei buffers e la rapidita'
                                ' di risposta quando vengono cambiati i parametri
                                ' del generatore.
Private Const TBuffMS# = 1000 * TBuff   ' Lunghezza dei buffers [ms].
'
Public bSigGenOn As Boolean     ' Flag di generazione segnale attiva.
'
Public bCallBack As Boolean     ' Flag di generazione del segnale con "callback".
'
Private Ic&(1 To 2)             ' Contatore dei campioni generati [campioni].
Private IcSweep&(1 To 2)        ' Contatore dei campioni generati nel modo "sweep" [campioni].
Public bChON(1 To 2) As Boolean ' Flags di canali attivi.
Public bSync(1 To 2) As Boolean ' Flags, mutuamente esclusivi, di canali sincronizzati.
'
                                ' Indici di precedenza per regolare la somma od il
                                ' prodotto dei segnali:
Public ICh_1C&                  '  indice del primo   canale da calcolare in GeneraCampioni16.
Public ICh_2C&                  '    "     "  secondo   "    "     "      "        "
Public ICh_Step&                ' ICh_Step = (ICh_2C - ICh_1C)
'
Public AmpIni#(1 To 2)          ' Ampiezze iniziale e
Public AmpFin#(1 To 2)          ' finale selezionate [Vu,p].
Public Const AmpMax# = 10#      ' Valor massimo assoluto ammesso per l' ampiezza [Vu,p].
Private AmpI#(1 To 2)           ' Ampiezze iniziale e
Private AmpF#(1 To 2)           ' finale selezionate [DigVal].
Private AmpD#(1 To 2)           ' Differenza ampiezze selezionate [DigVal].
'
Public Const V16Max# = 32767#   ' Massimo valore del segnale a 16 bits [DigVal].
Private Const V16Min# = -V16Max ' Minimo valore del segnale a 16 bits [DigVal].
'
Public FrqIni#(1 To 2)          ' Frequenze iniziale e
Public FrqFin#(1 To 2)          ' finale selezionate [Hz].
Public Const FreqMin# = 0.01    ' Frequenza minima ammessa [Hz].
Public FreqMax#                 ' Frequenza massima ammessa [Hz]; FreqMax = Fs / 2.
Private OmegaI#(1 To 2)         ' Pulsazioni iniziale e
Private OmegaF#(1 To 2)         ' finale selezionate [rad/campioni].
Private OmegaD#(1 To 2)         ' Differenza delle pulsazioni selezionate [rad/campioni].
'
Public TsRepPer#(1 To 2)    ' Periodo di ripetizione nei modi "burst" e "sweep" [s].
Private NcRepPer&(1 To 2)   ' Periodo di ripetizione nei modi "burst" e "sweep" [campioni].
Private NcRepPer_1#(1 To 2) ' Periodo - 1  di ripetizione nei modi "burst" e "sweep" [campioni].
'
Public TsBurst#(1 To 2)     ' Periodo del "burst" [s].
Private NcBurstT&(1 To 2)   ' Periodo del "burst" [campioni].
Private NcBurstT_1#(1 To 2) ' Periodo - 1 del "burst" [campioni].
'
Private LinOmF_OmI#(1 To 2) ' Per "sweep" lineare: (OmegaF - OmegaI) / (2# * NcRepPer_1) [rad/campioni].
Private LogOmF_OmI#(1 To 2) ' Per "sweep" logaritmico: Log(OmegaF / OmegaI) / NcRepPer_1 [1/campioni].
Private SweepDir&(1 To 2)   ' Direzione dello spazzolamento [campioni]:
                            '  SweepDir = 1  per direzione FrqIni -> FrqFin.
                            '  SweepDir = -1 per direzione FrqFin -> FrqIni.
'
Public bTrigOn(1 To 2) As Boolean   ' Flags di "burst" e "sweep" attivati da trigger.
Public bTrigOK(1 To 2) As Boolean   ' Flags di trigger attivo.
'
Public bAddMulCh(1 To 2) As Boolean ' Flags, mutuamente esclusivi, di abilitazione
                                    ' alla somma o prodotto dei segnali:
                                    '  bAddMulCh(1) = True: somma/prodotto del Ch. #2 sul Ch. #1.
                                    '  bAddMulCh(2) = True: somma/prodotto del Ch. #1 sul Ch. #2.
Public bAddCh(1 To 2) As Boolean    ' Flags di richiesta della somma:
                                    '  bAddCh = True per somma;
                                    '  bAddCh = False per prodotto.
'
Public PhaseD#(1 To 2)      ' Fase del segnale [Deg].
Private PhaseR#(1 To 2)     ' Fase del segnale (per Ch. #1 e' compensata da CompMux) [rad].
Private PhaseCh1R#          ' Fase del segnale del Ch. #1 senza compensazione da CompMux
                            ' (solo per modi "sweep") [rad].
'
Public NNodi&(1 To 2)       ' N di nodi - 1 per la costruzione grafica della forma d' onda.
Public TsPeriodoG#(1 To 2)  ' Periodo richiesto della forma d' onda grafica [s].  Questo
                            ' non e' il vero periodo dell' onda generata che e', invece,
                            ' uguale al multiplo di intervalli di campionamenti, NcPeriodoG(),
                            ' piu' vicino a TsPeriodoG(), i.e. NcPeriodoG/Fs = periodo vero.
Public NcPeriodoG&(1 To 2)  ' Periodo della forma d' onda grafica [campioni].
Public Type P_Type
    X() As Double   ' Ascisse della forma d' onda grafica [s].
    Y() As Double   ' Ordinate della forma d' onda grafica [1].
End Type
Public WaveN(1 To 2) As P_Type  ' Vettore dei punti (nodi) della forma d' onda grafica.
Public WaveS(1 To 2) As P_Type  ' Vettore dei punti (campioni) della forma d' onda
                                ' grafica interpolata.
'
Public bNorm(1 To 2) As Boolean ' Flags di normalizzazione attiva per la forma d' onda grafica.
'
                            ' Enumerazioni delle forme d' onda:
Public Const SineW& = 1     '  onda sinusoidale.
Public Const SquareW& = 2   '  onda quadra.
Public Const SawW& = 3      '  onda a denti di sega.
Public Const TriangW& = 4   '  onda triangolare.
Public Const NoiseW& = 5    '  rumore.
Public Const GraphW& = 6    '  onda come da grafico.
Public WaveForm&(1 To 2)    ' Forma d' onda corrente.
'
Public DutyCycle#(1 To 2)   ' Duty cycle dell' onda quadra (0 < DutyCycle < 1).
Public SimmL#(1 To 2)       ' Livello per la simmetria dell' onda triangolare (0 < SimmL < 1).
Private a1#(1 To 2)         ' Coefficienti
Private a2#(1 To 2)         ' per il calcolo
Private b2#(1 To 2)         ' dell' onda triangolare.
Private PhaseT#(1 To 2)     ' Fase per la centratura dell' onda triangolare [rad].
'
Private T01_O#(1 To 2)      ' Memoria dei valori T01 in Sub GeneraCampioni16.
'
                            ' Enumerazioni dei tipi di rumore:
Public Const White& = 1     '  rumore bianco a distribuzione uniforme.
Public Const WhiteGaus& = 2 '    "      "    "      "        normale.
Public Const Pink& = 3      '    "    rosa.
Public NoiseType&(1 To 2)   ' Tipo di rumore corrente.
'
                            ' Enumerazioni dei modi di funzionamento:
Public Const Run& = 1       '  continuo.
Public Const Burst& = 2     '  ad impulsi.
Public Const SweepLin& = 3  '  a spazzolamento di frequenza lineare.
Public Const SweepLog& = 4  '  "      "        "      "     logaritmico.
Public GMode&(1 To 2)       ' Modo di funzionamento corrente.
'
                            ' Enumerazioni del modo di spazzolamento in frequenza:
Public Const SweepR& = 1    '  spazzolamento a rampa (FrqIni -> FrqFin ...).
Public Const SweepT& = 2    '       "        bidirezionale (FrqIni -> FrqFin -> FrqIni ...).
Public SweepProf&(1 To 2)   ' Modo di spazzolamento corrente.
'
                            ' Enumerazioni delle interpolazioni delle forme d' onda:
Public Const PWLInt& = 1    '  lineare a tratti.
Public Const CubInt& = 2    '  spline cubica.
Public Const FourInt& = 3   '  con serie di Fourier.
Public GWaveInt&(1 To 2)    ' Interpolazione corrente.
'
Public KCal#                ' Coefficiente di calibrazione in uso [DigVal/Vu],
                            ' vedi nota 1 di frmSigGen.
'
Public CompMUX#             ' Costante di compensazione [s] del ritardo introdotto
                            ' dal multiplexer della scheda audio, vedi nota 3
                            ' di frmSigGen.
'
Private KRts#               ' KRts = -CompMUX * Fs; 0 <= Abs(KRts) <= 1 (per ritardo
                            ' interpolato linearmente) [campioni].
'
Public Tau#(1 To 2)         ' Tau(Ich) = IIf(ICh = 1, CompMUX, 0) ritardo del segnale
                            ' del canale #Ich [s].  Tau(2), attualmente sempre uguale
                            ' a zero, e' previsto per futuri sviluppi.
'
Private PhaseCM#            ' Correzione di fase (solo Ch. #1) per compensazione
                            ' del ritardo del multiplexer [rad].
'
Private Idum&               ' Seme della funzione Ran0(Idum).
'
Public Const NCanali& = 2   ' N di canali usati dalla scheda audio.
Public Const BitsPerSample& = 16    ' Suono a 16 bits.
'
Private NcBufferSize&       ' Dimensione dei vettori buffer [campioni].
Private OutData%()          ' Vettori [DigVal] dei dati da emettere (Integer, per suono a 16 bits).
Private OutDataC%()         ' Vettore temporaneo dei dati da emettere [DigVal].
Private NBytes&             ' N di bytes in OutDataC().
'
Private phwo&                       ' Handle del "waveform-audio output device".
Private WaveOutHdr() As WAVEHDR     ' WAVE header for sound data.
Private LenOutHeader&               ' Dimensione, in bytes, della struttura WAVEHDR.
'
Public FileINI$     ' File dei parametri dello strumento.
'
'-- Dichiarazioni, costanti e tipi per API: --------------------------------------------
'
Private Declare Function GetTickCount Lib "kernel32" () As Long
Private Declare Sub Sleep Lib "kernel32" (ByVal dwMilliseconds As Long)
'
Private Declare Sub CopyMemory Lib "kernel32" Alias "RtlMoveMemory" _
    (hpvDest As Any, hpvSource As Any, ByVal cbCopy As Long)
'
' Callback:
Private Const CALLBACK_NULL& = &H0              '  no callback.
Private Const CALLBACK_WINDOW& = &H10000        '  dwCallback is a HWND.
Private Const CALLBACK_TASK& = &H20000          '  dwCallback is a HTASK.
Private Const CALLBACK_FUNCTION& = &H30000      '  dwCallback is a FARPROC.
Private Const CALLBACK_TYPEMASK& = &H70000      '  callback type mask.
'
' Wave:
Private Const WAVE_MAPPER& = -1&
Private Const WAVE_FORMAT_PCM& = 1
Private Const WAVE_MAPPED& = &H4
Private Const WAVE_FORMAT_QUERY& = &H1
'
Private Type WAVEFORMATEX
    wFormatTag As Integer
    nChannels As Integer
    nSamplesPerSec As Long
    nAvgBytesPerSec As Long
    nBlockAlign As Integer
    wBitsPerSample As Integer
    cbSize As Integer
End Type
'
Private Type WAVEHDR
    lpData As Long
    dwBufferLength As Long
    dwBytesRecorded As Long
    dwUser As Long
    dwFlags As Long
    dwLoops As Long
    lpNext As Long
    Reserved As Long
End Type
'
' WaveOut:
Private Const MM_WOM_OPEN& = &H3BB  ' 955 waveform output.
Private Const MM_WOM_CLOSE& = &H3BC ' 956
Private Const MM_WOM_DONE& = &H3BD  ' 957
'
Private Const WOM_OPEN& = MM_WOM_OPEN
Private Const WOM_CLOSE& = MM_WOM_CLOSE
Private Const WOM_DONE& = MM_WOM_DONE
'
Private Const WHDR_DONE& = &H1      '  Done bit.
Private Const WHDR_BEGINLOOP& = &H4
Private Const WHDR_ENDLOOP& = &H8
Private Const WHDR_INQUEUE& = &H10
Private Const WHDR_PREPARED& = &H2
'
Private Const MMSYSERR_NOERROR& = 0 ' no error.
'
Private Declare Function waveOutOpen Lib "winmm.dll" _
    (lphWaveOut As Long, ByVal uDeviceID As Long, _
    lpFormat As WAVEFORMATEX, ByVal dwCallback As Long, _
    ByVal dwInstance As Long, ByVal dwFlags As Long) As Long
Private Declare Function waveOutReset Lib "winmm.dll" _
    (ByVal hWaveOut As Long) As Long
Private Declare Function waveOutClose Lib "winmm.dll" _
    (ByVal hWaveOut As Long) As Long
'
Private Declare Function waveOutPrepareHeader Lib "winmm.dll" _
    (ByVal hWaveOut As Long, lpWaveOutHdr As WAVEHDR, ByVal uSize As Long) As Long
Private Declare Function waveOutUnprepareHeader Lib "winmm.dll" _
    (ByVal hWaveOut As Long, lpWaveOutHdr As WAVEHDR, ByVal uSize As Long) As Long
'
Private Declare Function waveOutWrite Lib "winmm.dll" _
    (ByVal hWaveOut As Long, lpWaveOutHdr As WAVEHDR, ByVal uSize As Long) As Long
Public Sub ApplicaImpostazioni()
'
'   Questa routine e', normalmente, chiamata da frmImpostazioni
'   per rendere attivi i coefficienti di calibrazione, di ritardo
'   del multiplexer e della frequenza di campionamento:
'
    ' Calcolo il coefficiente di
    ' ritardo del segnale di Ch. #1:
    KRts = -CompMUX * Fs
    Tau(1) = CompMUX
    Tau(2) = 0#
'
    ' Aggiorno il grafico ed i parametri
    ' della forma d' onda sintetizzata:
    frmSigGen.DisegnaNodi 1
    frmSigGen.DisegnaInterpolazione 1
    frmSigGen.DisegnaNodi 2
    frmSigGen.DisegnaInterpolazione 2
'
    ' Applico i coefficienti
    ' alla generazione corrente:
    CalcolaParametri 1
    CalcolaParametri 2
'
'
'
End Sub
Public Function waveOutInizia(ByVal uDeviceSpID As Long) As Boolean
'
'
    Dim Ib&, lRet&, M$
    Dim FormatWaveOut As WAVEFORMATEX   ' Sound format.
'
    On Error GoTo waveOutInizia_ERR
'
    ' Azzero i contatori della
    ' Sub GeneraCampioni16:
    AzzeraContatori
'
    ' Calcolo il numero di campioni da emettere ...:
    NcBufferSize = CLng(TBuff * Fs) ' Dimensione dei buffers [campioni].
    ' ... ed imposto di conseguenza i vettori da "suonare":
    ReDim OutData(1 To NCanali, 1 To NcBufferSize, 1 To NBuff) As Integer
    ReDim OutDataC(1 To NCanali, 1 To NcBufferSize) As Integer
    NBytes = 2 * NCanali * NcBufferSize
'
    ' Preparo i vettori OutData()
    ' con il segnale da "suonare":
    For Ib = 1 To NBuff
        GeneraCampioni16 NcBufferSize, OutDataC()
        CopyMemory OutData(1, 1, Ib), OutDataC(1, 1), NBytes
    Next Ib
    GeneraCampioni16 NcBufferSize, OutDataC()   ' Pronto per il prossimo waveOutCallback
                                                ' o per waveOutEsegui.
'
    ' Imposto lo stato della scheda audio:
    With FormatWaveOut
        .wFormatTag = WAVE_FORMAT_PCM           ' uncompressed sound format.
        .nChannels = NCanali                    ' 1=mono; 2=stereo.
        .wBitsPerSample = BitsPerSample         ' bits per sample per channel.
        .nSamplesPerSec = CLng(Fs)              ' samples per second.
        .nBlockAlign = .nChannels * (.wBitsPerSample \ 8)
        .nAvgBytesPerSec = .nSamplesPerSec * .nBlockAlign
        .cbSize = 0
    End With
'
    ' Apro il collegamento con la scheda audio:
    If bCallBack Then
        lRet = waveOutOpen(phwo, uDeviceSpID, FormatWaveOut, _
               AddressOf waveOutCallback, True, CALLBACK_FUNCTION)
    Else
        lRet = waveOutOpen(phwo, uDeviceSpID, FormatWaveOut, 0, 0, 0)
    End If
    If (lRet <> MMSYSERR_NOERROR) Then
        MsgBox "waveOutOpen - Err: " & WaveERR(lRet), vbCritical, _
               " modSigGen: waveOutInizia_wOO"
        Call waveOutTermina_Err
        waveOutInizia = False
        Exit Function
    End If
    If (phwo = 0) Then
        M$ = "Il collegamento con la scheda audio non e' disponibile." & vbNewLine & vbNewLine
        M$ = M$ & "Si consiglia di chiudere tutti i programmi che potrebbero" & vbNewLine
        M$ = M$ & "usare la scheda audio o di riavviare il PC." & vbNewLine
        MsgBox M$, vbExclamation, " modSigGen: waveOutInizia_NCB"
        waveOutInizia = False
        Exit Function
    End If
'
    ' Preparo i buffers per l' emissione del segnale:
    ReDim WaveOutHdr(1 To NBuff) As WAVEHDR
    LenOutHeader = Len(WaveOutHdr(1))
    For Ib = 1 To NBuff
        WaveOutHdr(Ib).lpData = VarPtr(OutData(1, 1, Ib))
        WaveOutHdr(Ib).dwBufferLength = 2 * NCanali * NcBufferSize    ' [bytes].
        WaveOutHdr(Ib).dwFlags = 0&
        WaveOutHdr(Ib).dwLoops = 0&
        WaveOutHdr(Ib).dwBytesRecorded = 0&
        WaveOutHdr(Ib).dwUser = Ib
        WaveOutHdr(Ib).lpNext = 0&
        lRet = waveOutPrepareHeader(phwo, WaveOutHdr(Ib), LenOutHeader)
        If (lRet <> MMSYSERR_NOERROR) Then
            MsgBox "waveOutPrepareHeader " & Ib & " - Err: " & WaveERR(lRet), _
                    vbCritical, " modSigGen: waveOutInizia_wOPH"
            waveOutInizia = False
            Exit Function
        End If
    Next Ib
'
    ' Suono il buffer #1 e metto in coda gli altri:
    For Ib = 1 To NBuff
        lRet = waveOutWrite(phwo, WaveOutHdr(Ib), LenOutHeader)
    Next Ib
'
'
waveOutInizia_ERR:
    waveOutInizia = (Err.Number = 0)
'
'
'
End Function
Private Function waveOutCallback(ByVal hwi As Long, _
                                 ByVal uMsg As Long, _
                                 ByVal dwInstance As Long, _
                                 ByRef lpWaveOutHdr As WAVEHDR, _
                                 ByVal dwParam2 As Long) As Long
'
'   Routine per la gestione della procedura "CallBack".
'   Viene chiamata, con il parametro uMsg impostato a MM_WOM_DONE,
'   ogni volta che un buffer e' stato completamente "suonato";
'   eventuali altre chiamate, con uMsg impostato ad altri valori,
'   vengono ignorate:
'
    Dim lRet&, t0&, tElab&, Ib&
'
    t0 = GetTickCount
'
    If (uMsg = MM_WOM_DONE) And bSigGenOn Then
        Ib = lpWaveOutHdr.dwUser
        frmSigGen.lblIb.Caption = Ib
'
        ' Il buffer Ib e' stato completamente "suonato",
        ' lo riempio con nuovi dati ...:
        CopyMemory OutData(1, 1, Ib), OutDataC(1, 1), NBytes
        ' ... e lo rimetto in coda:
        lRet = waveOutWrite(phwo, WaveOutHdr(Ib), LenOutHeader)
'
        ' Preparo i dati del segnale per la
        ' prossima chiamata a questa funzione:
        GeneraCampioni16 NcBufferSize, OutDataC()
    End If
'
    ' Verifico che il tempo di riempimento dei buffers non
    ' sia stato superiore al massimo concesso di TBuffMS ms:
    tElab = GetTickCount - t0
    frmSigGen.lblFillTime.Caption = tElab
    If (TBuffMS <= tElab) Then
        bSigGenOn = False
        frmSigGen.lblFillTime.BackColor = vbRed
    End If
'
    ' Ritorno il valore di "successo":
    waveOutCallback = 1
'
'
'
End Function
Public Sub waveOutTermina()
'
'   Termina l' emissione del segnale dalla scheda audio
'   e la chiude:
'
    Dim Ib&, lRet&, t0&
'
    On Error Resume Next
'
    If (phwo <> 0) Then
        ' Fermo il suono corrente:
        waveOutReset (phwo)
'
        ' Libero i buffers di emissione dei dati ...:
        For Ib = 1 To NBuff
            t0 = GetTickCount
            Do  ' Attendo il rilascio del buffer Ib.
                DoEvents
            Loop Until ((WaveOutHdr(Ib).dwFlags And WHDR_DONE) = WHDR_DONE) _
                    Or (TBuffMS < (GetTickCount - t0))
'
            lRet = waveOutUnprepareHeader(phwo, WaveOutHdr(Ib), LenOutHeader)
            If (lRet <> MMSYSERR_NOERROR) Then
                MsgBox "waveOutUnPrepareHeader " & Ib & " - Err: " & WaveERR(lRet), _
                        vbCritical, " modSigGen: waveOutTermina_wOUH"
                Call waveOutTermina_Err
                Exit Sub
            End If
        Next Ib
'
        ' ... e chiudo il collegamento con la scheda audio:
        lRet = waveOutClose(phwo)
        If (lRet <> MMSYSERR_NOERROR) Then
            MsgBox "waveOutClose - Err: " & WaveERR(lRet), vbCritical, _
                   " modSigGen: waveOutTermina_wOC"
        End If
        phwo = 0
    End If
'
'
    If (Err.Number <> 0) Then Err.Clear
'
'
'
End Sub
Private Sub waveOutTermina_Err()
'
'   Questa routine, in caso di errori, prova a chiudere
'   in modo pulito il collegamento con la scheda audio:
'
    Dim lRet As Long
'
    On Local Error Resume Next
'
    If (phwo <> 0) Then
        lRet = waveOutReset(phwo)
        lRet = waveOutClose(phwo)
        phwo = 0
    End If
'
'
    If (Err.Number <> 0) Then Err.Clear
'
'
'
End Sub
Private Sub GeneraCampioni16(ByVal NSamples As Long, VOut() As Integer)
'
'   Genera NSamples campioni di un segnale ad ampiezza, frequenza
'   e forma d' onda voluta e li mette in VOut(1 To NSamples).
'   Il suono e' stereofonico a 16 bits.
'
'   Nota: per la costruzione delle forme d' onda quadra e triangolare
'         ho previsto due diversi metodi, da usare in alternativa ed
'         in funzione delle esigenze dell' utilizzatore (vedere com-
'         menti in loco).
'
    Dim Ich&, Itg&, N&, Amp#, Om#, DDC#, T01#, Vx#(1 To 2)
'
    On Error Resume Next
'
    For N = 1 To NSamples
        For Ich = ICh_1C To ICh_2C Step ICh_Step
            ' Operazioni relative al modo di funzionamento,
            ' come da quadro [Generator mode]:  <-----------------------------------
            If (GMode(Ich) = Run) Then
                ' Modo "Run":
                Amp = AmpI(Ich)
                Om = OmegaI(Ich) * CDbl(Ic(Ich)) + PhaseR(Ich)
'
            ElseIf (GMode(Ich) = Burst) Then
                ' Modo "Burst":
                If (NcRepPer(Ich) <= Ic(Ich)) Then
                    ' E' finito il periodo di ripetizione
                    ' e riazzero contatore e trigger:
                    Ic(Ich) = 0
                    bTrigOK(Ich) = False
                    frmSigGen.cmdTrigOK(Ich).BackColor = vbButtonFace
                    If bSync(Ich) Then Sincronizza Ich, Ic(Ich)
                End If
'
                If bTrigOn(Ich) And (Not bTrigOK(Ich)) Then
                    ' Sono in attesa del trigger:
                    Ic(Ich) = 0
                    Amp = 0#
                    Om = 0#
'
                Else
                    ' Sono nel periodo di ripetizione:
                    If bSync(Ich) Then
                        If (Ic(Ich) = 1) Then
                            Sincronizza Ich, Ic(Ich)    ' Serve quando il
                                                        ' trigger e' attivo.
                        End If
                    End If
'
                    If (Ic(Ich) < NcBurstT(Ich)) Then
                        ' Sono anche nel periodo di generazione del "burst":
                        Amp = AmpI(Ich) + AmpD(Ich) * CDbl(Ic(Ich)) / NcBurstT_1(Ich)
                        Om = OmegaI(Ich) * CDbl(Ic(Ich)) + PhaseR(Ich)
                    Else
                        ' Il periodo di generazione del "burst" e' finito
                        ' ma quello di ripetizione potrebbe continuare:
                        Amp = 0#
                        Om = 0#
                    End If
                End If
'
            ElseIf (GMode(Ich) = SweepLin) Then
                ' Modo "Sweep" lineare:
                If bTrigOn(Ich) And (Not bTrigOK(Ich)) Then
                    ' Sono in attesa del trigger:
                    Ic(Ich) = 0
                    IcSweep(Ich) = 0
                    SweepDir(Ich) = 1
                ElseIf bSync(Ich) Then
                    If (Ic(Ich) = 1) Then Sincronizza Ich, Ic(Ich)
                End If
'
                If (NcRepPer(Ich) <= Ic(Ich)) Then
                    ' E' finito il periodo di ripetizione
                    ' e riazzero contatore e trigger:
                    Ic(Ich) = 0
                    bTrigOK(Ich) = False
'
                    If (SweepProf(Ich) = SweepR) Then
                        IcSweep(Ich) = 0
                        SweepDir(Ich) = 1
                    Else
                        SweepDir(Ich) = -SweepDir(Ich)
                    End If
'
                    frmSigGen.cmdTrigOK(Ich).BackColor = vbButtonFace
                    If bSync(Ich) Then Sincronizza Ich, Ic(Ich)
                End If
'
                ' Calcolo ampiezza ...:
                Amp = AmpI(Ich) + AmpD(Ich) * CDbl(IcSweep(Ich)) / NcRepPer_1(Ich)
'
                ' ... e pulsazione istantanea:
                If (Ich = 1) Then
                    ' La correzione di fase e' funzione della frequenza:
                    PhaseCM = KRts * (OmegaI(1) + OmegaD(1) * CDbl(IcSweep(1)) _
                            / NcRepPer_1(1))
                    PhaseR(1) = PhaseCM + PhaseCh1R ' Fase totale.
                End If
                Om = (OmegaI(Ich) + LinOmF_OmI(Ich) * CDbl(IcSweep(Ich))) _
                   * CDbl(IcSweep(Ich)) _
                   + PhaseR(Ich)
'
                IcSweep(Ich) = IcSweep(Ich) + SweepDir(Ich)
'
            ElseIf (GMode(Ich) = SweepLog) Then
                ' Modo "Sweep" logaritmico:
                If bTrigOn(Ich) And (Not bTrigOK(Ich)) Then
                    ' Sono in attesa del trigger:
                    Ic(Ich) = 0
                    IcSweep(Ich) = 0
                    SweepDir(Ich) = 1
                ElseIf bSync(Ich) Then
                    If (Ic(Ich) = 1) Then Sincronizza Ich, Ic(Ich)
                End If
'
                If (NcRepPer(Ich) <= Ic(Ich)) Then
                    ' E' finito il periodo di ripetizione
                    ' e riazzero contatore e trigger:
                    Ic(Ich) = 0
                    bTrigOK(Ich) = False
'
                    If (SweepProf(Ich) = SweepR) Then
                        IcSweep(Ich) = 0
                        SweepDir(Ich) = 1
                    Else
                        SweepDir(Ich) = -SweepDir(Ich)
                    End If
'
                    frmSigGen.cmdTrigOK(Ich).BackColor = vbButtonFace
                    If bSync(Ich) Then Sincronizza Ich, Ic(Ich)
                End If
'
                ' Calcolo ampiezza ...:
                Amp = AmpI(Ich) + AmpD(Ich) * CDbl(IcSweep(Ich)) / NcRepPer_1(Ich)
'
                ' ... e pulsazione istantanea:
                If (Ich = 1) Then
                    ' La correzione di fase e' funzione della frequenza:
                    PhaseCM = KRts * (OmegaI(1) + OmegaD(1) * CDbl(IcSweep(1)) _
                            / NcRepPer_1(1))
                    PhaseR(1) = PhaseCM + PhaseCh1R
                End If
                Om = (OmegaI(Ich) / LogOmF_OmI(Ich)) _
                   * (Exp(CDbl(IcSweep(Ich)) * LogOmF_OmI(Ich)) - 1#) _
                   + PhaseR(Ich)
'
                IcSweep(Ich) = IcSweep(Ich) + SweepDir(Ich)
            End If
'
            ' Modello la forma d' onda,
            ' come da quadro [Wave form]:       <-----------------------------------
            If (WaveForm(Ich) = SineW) Then
                ' Onda sinusoidale:
                Vx(Ich) = Amp * Sin(Om)
'
            ElseIf (WaveForm(Ich) = SquareW) Then
                ' Onda quadra:
                T01 = Om / PI2          ' Operazione di MODULO(Om, PI2)/PI2; >-
                DDC = T01 - T01_O(Ich)  '                                      |
                T01_O(Ich) = T01        '                                      |
                T01 = T01 - Int(T01)    ' 0 <= MODULO(Om, PI2)/PI2 < 1.      <-
'
                ' Versione con fronti d' onda piu'
                ' ripidi ma che puo', in certe condizioni,
                ' presentare un notevole "jitter" ed una
                ' risposta discontinua ai cambi di fase:
'                If (T01 <= DutyCycle(ICh)) Then
'                    Vx(ICh) = Amp
'                Else
'                    Vx(ICh) = -Amp
'                End If
'
                ' Versione con fronti d' onda interpolati
                ' (e quindi meno ripidi) che consente una
                ' buona minimizzazione del "jitter" ed
                ' una risposta continua ai cambi di fase:
                If (T01 < DDC) Then
                    Vx(Ich) = Amp * T01 / DDC
                ElseIf (T01 < DutyCycle(Ich) - DDC) Then
                    Vx(Ich) = Amp
                ElseIf (T01 < DutyCycle(Ich) + DDC) Then
                    Vx(Ich) = Amp * (DutyCycle(Ich) - T01) / DDC
                ElseIf (T01 < 1# - DDC) Then
                    Vx(Ich) = -Amp
                Else
                    Vx(Ich) = Amp * (T01 - 1#) / DDC
                End If
'
            ElseIf (WaveForm(Ich) = SawW) Then
                ' Onda a dente di sega:
                T01 = Om / PI2          ' Operazione di MODULO(Om, PI2)/PI2; >-
                DDC = T01 - T01_O(Ich)  '                                      |
                T01_O(Ich) = T01        '                                      |
                T01 = T01 - Int(T01)    ' 0 <= MODULO(Om, PI2)/PI2 < 1.      <-
'
                ' Versione con fronte di discesa piu'
                ' ripido, ma che puo', in certe condizioni,
                ' presentare un notevole "jitter" ed una
                ' risposta discontinua ai cambi di fase:
'                Vx(ICh) = Amp * (2# * T01 - 1#)
'
                ' Versione con fronte di discesa interpolato
                ' (e quindi meno ripido) che consente una
                ' buona minimizzazione del "jitter" ed
                ' una risposta continua ai cambi di fase:
                If (T01 < DDC) Then
                    Vx(Ich) = -Amp * T01 / DDC
                ElseIf (T01 < 1# - DDC) Then
                    Vx(Ich) = Amp * (2# * T01 - 1#) / (1# - 2# * DDC)
                Else
                    Vx(Ich) = Amp * (1# - T01) / DDC
                End If
'
            ElseIf (WaveForm(Ich) = TriangW) Then
                ' Onda triangolare:
                T01 = (Om + PhaseT(Ich)) / PI2  ' Operazione di MODULO(Om + PhaseT(ICh), PI2)/PI2;
                T01 = T01 - Int(T01)            ' 0 <= MODULO(Om + PhaseT(ICh), PI2)/PI2 < 1.
                If (T01 < SimmL(Ich)) Then
                    Vx(Ich) = Amp * (a1(Ich) * T01 + 1#)
                Else
                    Vx(Ich) = Amp * (a2(Ich) * T01 + b2(Ich))
                End If
'
            ElseIf (WaveForm(Ich) = NoiseW) Then
                ' Rumore:
                If (NoiseType(Ich) = White) Then
                    Vx(Ich) = Amp * WhiteNoise(Idum)
                ElseIf (NoiseType(Ich) = WhiteGaus) Then
                    Vx(Ich) = Amp * WhiteGaussNoise(Idum)
                ElseIf (NoiseType(Ich) = Pink) Then
                    Vx(Ich) = Amp * PinkNoise(Ich, Idum)
                End If
'
            ElseIf (WaveForm(Ich) = GraphW) Then
                ' Onda come da grafico:
                Itg = Ic(Ich) Mod NcPeriodoG(Ich)
                Vx(Ich) = Amp * WaveS(Ich).Y(Itg)
                If bSync(Ich) Then
                    If (Itg = 0) Then
                        If (GMode(Ich) = Run) Then Sincronizza Ich, Itg
                    End If
                End If
            End If
'
            If bAddMulCh(Ich) Then
                If bAddCh(Ich) Then
                    ' Sommo, sul segnale del canale ICh,
                    ' il segnale dell' altro canale:
                    Vx(Ich) = Vx(1) + Vx(2)
                Else
                    ' Il segnale del canale ICh e'
                    ' il prodotto dei due segnali:
                    Vx(Ich) = (Vx(1) * Vx(2)) / KCal    ' Per preservare la correttezza
                                                        ' dimensionale di questa espres-
                                                        ' sione, le AmpIni (e AmpFin) di
                                                        ' uno dei due canali sono da con-
                                                        ' siderare come coefficienti adi-
                                                        ' mensionali.
                End If
            End If
'
            ' Limito il segnale di uscita all 'intervallo
            ' di valori ammesso dalla risoluzione a 16 bits:
            If (Vx(Ich) < V16Min) Then Vx(Ich) = V16Min
            If (V16Max < Vx(Ich)) Then Vx(Ich) = V16Max
'
            If bChON(Ich) Then
                ' Il canale ICh o e' attivo o e'
                ' premuto il suo pulsante [Gate].
                VOut(Ich, N) = CInt(Vx(Ich))
            Else
                ' Il canale ICh e' spento e non e'
                ' premuto il suo pulsante [Gate]:
                VOut(Ich, N) = 0
            End If
'
            Ic(Ich) = Ic(Ich) + 1
        Next Ich
    Next N
'
'
    If (Err.Number <> 0) Then Err.Clear
'
'
'
End Sub
Public Sub CalcolaParametri(ByVal Ich As Integer)
'
'
    ' Ampiezze iniziale e finale
    ' in unita' [DigVal]:
    AmpI(Ich) = KCal * AmpIni(Ich)
    AmpF(Ich) = KCal * AmpFin(Ich)
    AmpD(Ich) = AmpF(Ich) - AmpI(Ich)
'
    ' Pulsazioni iniziale e finale
    ' in unita' [rad/campioni]:
    OmegaI(Ich) = PI2 * FrqIni(Ich) / Fs
    OmegaF(Ich) = PI2 * FrqFin(Ich) / Fs
    OmegaD(Ich) = OmegaF(Ich) - OmegaI(Ich)
'
    ' Coefficienti e fase per il calcolo
    ' dell' onda triangolare:
    a1(Ich) = -2# / SimmL(Ich)
    a2(Ich) = -2# / (SimmL(Ich) - 1#)
    b2(Ich) = (SimmL(Ich) + 1#) / (SimmL(Ich) - 1#)
    PhaseT(Ich) = -PI_2                                                 ' Scostamento di fase fisso.
    PhaseT(Ich) = PhaseT(Ich) - PI2 * (0.75 - (SimmL(Ich) + 1#) / 2#)   ' Scostamento di fase funzione
                                                                        ' della simmetria richiesta.
'
    ' N di campioni per il periodo di
    ' ripetizione di "Burst" e "Sweep":
    NcRepPer(Ich) = CLng(TsRepPer(Ich) * Fs)
    NcRepPer_1(Ich) = CDbl(CLng(TsRepPer(Ich) * Fs) - 1)
'
    ' N di campioni per il periodo di "Burst":
    NcBurstT(Ich) = CLng(TsBurst(Ich) * Fs)
    NcBurstT_1(Ich) = CDbl(CLng(TsBurst(Ich) * Fs) - 1)
'
    ' Coefficienti delle pulsazioni per "sweep":
    LinOmF_OmI(Ich) = OmegaD(Ich) / (2# * NcRepPer_1(Ich))
    LogOmF_OmI(Ich) = Log(OmegaF(Ich) / OmegaI(Ich)) / NcRepPer_1(Ich)
    If (LogOmF_OmI(Ich) = 0#) Then LogOmF_OmI(Ich) = 0.000000001
'
    ' Parametri della fase:
    If (Ich = 1) Then
        ' Calcolo la compensazione di fase
        ' per il ritardo del multiplexer:
        If (GMode(1) = Burst) Then
            ' In modo "Burst" e' poco utile e darebbe, comunque,
            ' discontinuita' troppo evidenti con onde quadre ed a rampa:
            PhaseCM = 0#
        Else
            PhaseCM = KRts * PI2 * FrqIni(1) / Fs
        End If
        PhaseCh1R = PhaseD(1) * PI / 180#   ' Solo per "sweep".
        PhaseR(1) = PhaseCM + PhaseCh1R     ' E' la fase totale del Ch. #1.
    Else
        PhaseR(2) = PhaseD(2) * PI / 180#
    End If
'
'
'
End Sub
Private Sub LeggiParametri()
'
'   Legge, dal file SigGen.ini, i parametri di impostazione:
'
    Fs = CDbl(GetIniString(FileINI$, "Impostazioni:", "Fs", 11025))
'
    MixerSpI = CLng(GetIniString(FileINI$, "Impostazioni:", "MixerSpI", "1"))
    MixerSpI = MAX0(MIN0(MixerSpI, NMixSp), 1)
'
    KCal = Val(ValC2D(GetIniString(FileINI$, "Impostazioni:", "KCal", -9610)))
    CompMUX = Val(ValC2D(GetIniString(FileINI$, "Impostazioni:", "CompMUX", 1# / 48000)))
'
    bCallBack = CBool(GetIniString(FileINI$, "Impostazioni:", "Callback", False))
'
'
'
End Sub
Public Sub Main()
'
'
    On Error Resume Next    ' Gestisce gli errori che si possono verificare durante
                            ' il frmSigGen_Load (e.g. mancanza di scheda audio).
'
    ' Verifico la disponibilita' di almeno una scheda audio:
    If (Not CercaSchedeAudio) Then
        MsgBox "Questo PC non ha schede audio.", _
               vbCritical, " modDecoToni: Main"
        Exit Sub
    End If
'
    ' Cerco i Mixers, con una destinazione di tipo SPEAKERS:
    CercaMixersSp
    If (Not CercaMixersSp) Then
        MsgBox "La ricerca dei mixers audio ha" & vbNewLine & _
               "trovato una condizione imprevista." & vbNewLine & vbNewLine & _
               "Per la selezione dei canali di ingresso" & vbNewLine & _
               "provare ad usare il ""Volume Control""" & vbNewLine & _
               "di Windows." & vbNewLine, _
               vbCritical, " modDecoToni: Main_CMS"
    End If
'
    FileINI$ = App.Path & "\SigGen.ini"
'
    ' Inizializzo il generatore di numeri casuali:
    Idum = -10
    Call WhiteGaussNoise(Idum)
'
    LeggiParametri
'
    frmSigGen.Show
'
    ' Calcolo le compensazioni del
    ' ritardo del multiplexer:
    ApplicaImpostazioni
'
    ' "Splash Form" iniziale:
    'frmCopyright.Apri frmSigGen, vbModeless, 4
'
'
    If (Err.Number <> 0) Then Err.Clear
'
'
'
End Sub
Private Sub Sincronizza(ByVal Ich As Long, ByVal ItSync As Long)
'
'   Sincronizzazione dei canali durante la generazione dei
'   segnali (Sub GeneraCampioni16).
'   ICh e' il canale che comanda il sincronismo; ItSync e'
'   il valore di contatore da attribuire all' altro canale:
'
    If (Ich = 1) And (Not bTrigOn(2)) Then
        Ic(2) = ItSync
        IcSweep(2) = 0
        SweepDir(2) = 1
'
    ElseIf (Ich = 2) And (Not bTrigOn(1)) Then
        Ic(1) = ItSync
        IcSweep(1) = 0
        SweepDir(1) = 1
    End If
'
'
'
End Sub
Public Sub AzzeraContatori()
'
'   Re-inizializza i contatori di GeneraCampioni16, quando si
'   cambia il "Generator mode"; opera su entrambi i canali per
'   ottenere, eventualmente, la sincronizzazione dei segnali:
'
    Ic(1) = 0
    Ic(2) = 0
    IcSweep(1) = 0
    IcSweep(2) = 0
    SweepDir(1) = 1
    SweepDir(2) = 1
'
    T01_O(1) = 0#
    T01_O(2) = 0#
'
'
'
End Sub
Public Sub CalcolaOndaG(ByVal Ich As Integer, ByRef YMed As Double)
'
'   Calcola i campioni dell' onda grafica in funzione
'   del tipo di interpolazione richiesta:
'
    Dim N&, TH#, YAbsMax#, PeriodoV#
'
    NcPeriodoG(Ich) = CLng(TsPeriodoG(Ich) * Fs)
    ReDim WaveS(Ich).X(0 To NcPeriodoG(Ich)), WaveS(Ich).Y(0 To NcPeriodoG(Ich))
'
    ' Calcolo l' interpolazione selezionata:
    PeriodoV = NcPeriodoG(Ich) / Fs
    TH = PhaseD(Ich) * PeriodoV / 360#  ' Ritardo/anticipo corrispondente
                                        ' alla fase richiesta [s].
    If (GWaveInt(Ich) = PWLInt) Then
        InterpolaL Ich, WaveN(), WaveS(), Tau(Ich) - TH, YMed, YAbsMax
    ElseIf (GWaveInt(Ich) = CubInt) Then
        InterpolaC Ich, WaveN(), WaveS(), Tau(Ich) - TH, YMed, YAbsMax
    ElseIf (GWaveInt(Ich) = FourInt) Then
        InterpolaF Ich, WaveN(), WaveS(), Tau(Ich) - TH, YMed, YAbsMax
    End If
'
    If bNorm(Ich) And (YAbsMax <> 0) Then
        ' Normalizzo ad 1 il valor massimo assoluto:
        For N = 0 To NcPeriodoG(Ich)
            WaveS(Ich).Y(N) = WaveS(Ich).Y(N) / YAbsMax
        Next N
    End If
'
'
'
End Sub
Public Function waveOutEsegui() As Boolean
'
'   Routine di generazione dei segnali (alternativa a waveOutCallback);
'   consuma piu' risorse di CPU perche' continua, ininterrottamente, ad
'   eseguire il codice del ciclo Do ... Loop While bSigGenOn:
'
    Dim Ib&, TBuffErr&, tElab&, t0&, lRet&
'
    On Error GoTo waveOutEsegui_ERR
'
    TBuffErr = MAX0(10 * NBuff * TBuffMS, 500)  ' massimo tempo di attesa per
                                                ' il riempimento dei buffers [ms].
'
    Ib = 1
    Do
        frmSigGen.lblIb.Caption = Ib
'
        t0 = GetTickCount   ' [ms]; uso GetTickCount perche' e' piu' veloce,
                            ' anche se meno preciso, di timeGetTime.
'
        Do  ' Attendo lo svuotamento del buffer Ib.
            If (TBuffErr < (GetTickCount - t0)) Then
                Err.Raise 1001, " Acquisizione", "Il sistema non risponde."
            End If
        Loop Until (WaveOutHdr(Ib).dwFlags And WHDR_DONE)
'
        t0 = GetTickCount
'
        ' Il buffer Ib e' stato completamente "suonato",
        ' lo riempio con nuovi dati ...:
        CopyMemory OutData(1, 1, Ib), OutDataC(1, 1), NBytes
        ' ... e lo rimetto in coda:
        lRet = waveOutWrite(phwo, WaveOutHdr(Ib), LenOutHeader)
'
        ' Preparo i dati del segnale per la
        ' prossima chiamata a questa funzione:
        GeneraCampioni16 NcBufferSize, OutDataC()
'
        ' Calcolo l' indice del buffer successivo:
        Ib = Ib + 1
        If (NBuff < Ib) Then Ib = 1
'
        ' Verifico che il tempo di riempimento dei buffers non
        ' sia stato superiore al massimo concesso di TBuffMS ms:
        tElab = GetTickCount - t0
        frmSigGen.lblFillTime.Caption = tElab
        If (TBuffMS <= tElab) Then
            'bSigGenOn = False
            frmSigGen.lblFillTime.BackColor = vbRed
        End If
'
        DoEvents    ' Unico DoEvents nel ciclo di generazione.
    Loop While bSigGenOn
'
'
waveOutEsegui_ERR:
    If (Err.Number <> 0) Then
        Dim M$
        M$ = "Errore " & Str$(Err.Number) & vbNewLine
        M$ = M$ & Err.Description
        MsgBox M$, vbCritical, " modSigGen: waveOutEsegui"
        waveOutEsegui = False
'
    Else
        waveOutEsegui = True
    End If
'
'
'
End Function
